/***********************
 *
 * ScanData.hh
 *
 * Kenneth McCallum
 * Columbia University
 *
 ***********************/

#ifndef __SCANDATA_EB_MCC_HH__
#define __SCANDATA_EB_MCC_HH__

#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <cmath>
#include <algorithm>
#include <gsl/gsl_sf_gamma.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

class scanData {

private:

  /************** Control variants **********/
  double maf_max; // Maximum MAF for variants included in study

  /******************* Data *****************/
  // Observational Data
  int ** genotype; // Genotype data. Rows correspond to individual subjects, columns to variant loci
  int * phenotype; // The observed phenotype per individual
  int * coordinate; // Coordinates of variant loci
  int NumSubjects; // Number of subjects in the data set
  int NumVariants; // Number of variants in the data set

  // Reduced Data
  int * Y; // Number of case individuals with minor allele at each variant loci
  int * N; // Number of case and control individuals with minor allele at each variant loci
  int * C; // Genomic coordinates of variant loci that pass MAF filter
  double * MAF; // Minor allele frequency for each variant loci
  int NumCases; // Number of case individuals
  int NumControl; // Number of control individuals
  int NumVarRet; // Number of variants that pass MAF filter

  int * Yu; // Y values for unique (N,Y) pairs
  int * Nu; // N values for unique (N,Y) pairs
  int * Map; // Map from C to Yu and Nu
  double * NcY; // log of N choose Y for unique pairs
  int NumUnique; // Number of unique pairs

  int * concat, * backup; // Used in finding unique data points

  /***************** Windows ***************/
  int * windowSizes; // The sizes of sliding windows used
  int NumSizes; // Number of different window sizes used
  int ** winStart; // Index of first variant in each window (second index) for each window size (first index)
  int ** winEnd; // Index of last variant in each window (second index) for each window size (first index)
  int * NumWindows; // Number of windows for each window size

  /**************** Random number generation **********/
  gsl_rng * rgen; // Random number generator from GNU GSL
  const gsl_rng_type * T;

  /************** Data manipulation methods ************/
  void reduce_data( ); // Calculate Yu, Nu, and Map from genotype and phenotype
  void calc_windows( ); // Calculate window boundaries

public:

  /**************** Constructor and destructor **********/
  scanData( const char *, const char *, const char *, int *, int, double, int, int, int ); // Loads data from file const char *
  ~scanData( ); // destructor

  /**************** Permutation methods ******************/
  void permutation_test_rows( );
  void permutation_test_columns( );

  /**************** Access Methods *********/
  int yu( int ); // Access unique Y values
  int nu( int ); // Access unique N values
  int map( int ); // Access Map values
  int c( int ); // Access C values
  double maf( int ); // Access MAF values
  double ncy( int ); // Access NcY values
  int numVariants( ); // Access NumVariants
  int numVarRet( ); // Access NumVarRet
  int numUnique( ); // Access NumUnique
  int numSizes( ); // Access NumSizes
  int WindowSizes( int ); // Access windowSizes values
  int WinStart( int, int ); // Access winStart values
  int WinEnd( int, int ); // Access winEnd values
  int numWindows( int ); // Access NumWindows values
  int numCases( ){ return NumCases; }
  int numControls( ){ return NumControl; }
  int genotypeVal( int i , int j ){ return genotype[i][j]; }
  int phenotypeVal( int i ) { return phenotype[i]; }
};

#endif
