#define THRESHOLD_MIN 1e-12
#define MAX_ITER_MIN 500
#define THRESHOLD_EM 1e-12
#define MAX_ITER_EM 500
#define THRESHOLD_PI 1e-6
#define MAX_ITER_PI 100

#include "ScanModel.hh"
#include <ctime>

using namespace std;

double scanModel::calculate_Q ( const gsl_vector * x, void * param ) {

  scanModel * mo = (scanModel *)param;

  // Check that parameter values are within acceptable range
  if ( (exp(gsl_vector_get(x,0)) < 1e-6) | (exp(gsl_vector_get(x,1)) < 1e-6) | (exp(gsl_vector_get(x,2)) < 1e-6) | (exp(gsl_vector_get(x,3)) < 1e-6) ) {
    return DBL_MAX; // Returning DBL_MAX guarentees minimization algorithm rejects the input point
  } else if ((exp(gsl_vector_get(x,0))>1e6) | (exp(gsl_vector_get(x,1))>1e6) | (exp(gsl_vector_get(x,2))>1e6) | (exp(gsl_vector_get(x,3))>1e6) ) {
    return DBL_MAX;
  } else {
    mo->Alpha[0] = exp(gsl_vector_get(x,0));
    mo->Alpha[1] = exp(gsl_vector_get(x,1));
    mo->Beta[0] = exp(gsl_vector_get(x,2));
    mo->Beta[1] = exp(gsl_vector_get(x,3));
  }

  // Calculate quantities constant across observations
  mo->ln_beta_func_0 = gsl_sf_lnbeta( mo->Alpha[0], mo->Beta[0] );
  mo->ln_beta_func_1 = gsl_sf_lnbeta( mo->Alpha[1], mo->Beta[1] );
  mo->ln_pi[0] = log(mo->Pi[0]);
  mo->ln_1_minus_pi[0] = log(1-mo->Pi[0]);

  // Calculate quantities for unique values of observations
  for ( int i = 0 ; i < mo->sd->numUnique( ) ; i++ ) {
    mo->ln_beta_post_0[i] = gsl_sf_lnbeta( mo->Alpha[0] + mo->sd->yu(i), mo->Beta[0] + mo->sd->nu(i) - mo->sd->yu(i) );
    mo->ln_beta_post_1[i] = gsl_sf_lnbeta( mo->Alpha[1] + mo->sd->yu(i), mo->Beta[1] + mo->sd->nu(i) - mo->sd->yu(i) );
  }

  // Calculate Q value
  double Q = 0;
  for ( int i = 0 ; i < mo->sd->numVarRet( ) ; i++ ) {
    Q += mo->psi[i] * ( mo->sd->ncy(mo->sd->map(i)) + mo->ln_pi[0] + mo->ln_beta_post_0[mo->sd->map(i)] - mo->ln_beta_func_0 );
    Q += (1-mo->psi[i]) * ( mo->sd->ncy(mo->sd->map(i)) + mo->ln_1_minus_pi[0] + mo->ln_beta_post_1[mo->sd->map(i)] - mo->ln_beta_func_1 );
  }

  return -Q;

}

double scanModel::model_1_objective ( const gsl_vector * x, void * param ) {

  scanModel * mo = (scanModel *)param;

  // Check that parameter values are within acceptable range
  if ( (exp(gsl_vector_get(x,0)) < 1e-6) | (exp(gsl_vector_get(x,1)) < 1e-6) | (exp(gsl_vector_get(x,2)) < 1e-6) | (exp(gsl_vector_get(x,3)) < 1e-6) ) {
    return DBL_MAX; // Returning DBL_MAX guarentees minimization algorithm rejects the input point
  } else if ((exp(gsl_vector_get(x,0))>1e6) | (exp(gsl_vector_get(x,1))>1e6) | (exp(gsl_vector_get(x,2))>1e6) | (exp(gsl_vector_get(x,3))>1e6) ) {
    return DBL_MAX;
  } else {
    mo->Alpha_m1[0] = exp(gsl_vector_get(x,0));
    mo->Alpha_m1[1] = exp(gsl_vector_get(x,1));
    mo->Beta_m1[0] = exp(gsl_vector_get(x,2));
    mo->Beta_m1[1] = exp(gsl_vector_get(x,3));
  }
  double mu1, mu0;
  mu1 = mo->Alpha_m1[1] / ( mo->Alpha_m1[1] + mo->Beta_m1[1] );
  mu0 = mo->Alpha_m1[0] / ( mo->Alpha_m1[0] + mo->Beta_m1[0] );
  if ( mu0 > mu1 ) {
    return DBL_MAX;
  }

  // Calculate quantities constant across observations
  mo->ln_beta_func_0_m1 = gsl_sf_lnbeta( mo->Alpha_m1[0], mo->Beta_m1[0] );
  mo->ln_beta_func_1_m1 = gsl_sf_lnbeta( mo->Alpha_m1[1], mo->Beta_m1[1] );

  // Calculate quantities for unique values of observations
  for ( int i = 0 ; i < mo->sd->numUnique( ) ; i++ ) {
    mo->ln_beta_post_0_m1[i] = gsl_sf_lnbeta( mo->Alpha_m1[0] + mo->sd->yu(i), mo->Beta_m1[0] + mo->sd->nu(i) - mo->sd->yu(i) );
    mo->ln_beta_post_1_m1[i] = gsl_sf_lnbeta( mo->Alpha_m1[1] + mo->sd->yu(i), mo->Beta_m1[1] + mo->sd->nu(i) - mo->sd->yu(i) );
  }

  double LL = 0;

  for ( int i = 0 ; i < mo->win_l ; i++ ) {
    LL += mo->ln_beta_post_0_m1[mo->sd->map(i)] - mo->ln_beta_func_0_m1;
  }
  for ( int i = mo->win_l ; i <= mo->win_u ; i++ ) {
    LL += mo->ln_beta_post_1_m1[mo->sd->map(i)] - mo->ln_beta_func_1_m1;
  }
  for ( int i = (mo->win_u+1) ; i < mo->sd->numVarRet( ) ; i++ ) {
    LL += mo->ln_beta_post_0_m1[mo->sd->map(i)] - mo->ln_beta_func_0_m1;
  }

  return -LL;

}

void scanModel::estimate_pi ( int lower, int upper ) {

  double N;

  // Window
  Pi[1] = 0;
  N = 0;
  for ( int i = lower ; i <= upper ; i++ ) {
    Pi[1] += psi[i];
    N += 1;
  }
  if ( N > 0 ) {
    Pi[1] /= N;
  } else {
    Pi[1] = 0.5;
  }

  // Non window or null
  Pi[0] = 0;
  N = 0;
  for ( int i = 0 ; i < lower ; i++ ) {
    Pi[0] += psi[i];
    N += 1;
  }
  for ( int i = (upper+1) ; i < sd->numVarRet( ) ; i++ ) {
    Pi[0] += psi[i];
    N += 1;
  }
  if ( N > 0 ) {
    Pi[0] /= N;
  } else {
    Pi[0] = 0.5;
  }

  // Ensure non-zero values for pi and 1-pi
  if ( Pi[0] < 0.001 ) {
    Pi[0] = 0.001;
  } else if ( Pi[0] > 0.999 ) {
    Pi[0] = 0.999;
  }
  if ( Pi[1] < 0.001 ) {
    Pi[1] = 0.001;
  } else if ( Pi[1] > 0.999 ) {
    Pi[1] = 0.999;
  }

}

void scanModel::expectation ( int lower, int upper ) {

  double first, second;
  ln_pi[0] = log(Pi[0]);
  ln_pi[1] = log(Pi[1]);
  ln_1_minus_pi[0] = log(1-Pi[0]);
  ln_1_minus_pi[1] = log(1-Pi[1]);

  // Calculate quantities constant across observations
  ln_beta_func_0 = gsl_sf_lnbeta( Alpha[0], Beta[0] );
  ln_beta_func_1 = gsl_sf_lnbeta( Alpha[1], Beta[1] );

  // Calculate quantities for unique values of observations
  for ( int i = 0 ; i < sd->numUnique( ) ; i++ ) {
    ln_beta_post_0[i] = gsl_sf_lnbeta( Alpha[0] + sd->yu(i), Beta[0] + sd->nu(i) - sd->yu(i) );
    ln_beta_post_1[i] = gsl_sf_lnbeta( Alpha[1] + sd->yu(i), Beta[1] + sd->nu(i) - sd->yu(i) );
  }

  for ( int i = 0 ; i < lower ; i++ ) {
    first = exp(ln_pi[0] + ln_beta_post_0[sd->map(i)] - ln_beta_func_0);
    second = exp(ln_1_minus_pi[0] + ln_beta_post_1[sd->map(i)] - ln_beta_func_1);
    psi[i] = first / ( first + second );
  }
  for ( int i = lower ; i <= upper ; i++ ) {
    first = exp(ln_pi[1] + ln_beta_post_0[sd->map(i)] - ln_beta_func_0);
    second = exp(ln_1_minus_pi[1] + ln_beta_post_1[sd->map(i)] - ln_beta_func_1);
    psi[i] = first / ( first + second );
  }
  for ( int i = (upper+1) ; i < sd->numVarRet() ; i++ ) {
    first = exp(ln_pi[0] + ln_beta_post_0[sd->map(i)] - ln_beta_func_0);
    second = exp(ln_1_minus_pi[0] + ln_beta_post_1[sd->map(i)] - ln_beta_func_1);
    psi[i] = first / ( first + second );
  }
}

void scanModel::estimate_ab ( ) {

  size_t iter = 0;
  int status;
  double size;

  for ( int i = 0 ; i < 2 ; i++ ) {
    gsl_vector_set( minimizer_x, i, log(Alpha[i]) );
    gsl_vector_set( minimizer_x, (i+2), log(Beta[i]) );
  }

  gsl_vector_set_all( step, 1.0 );
  func.n = 4;
  func.f = calculate_Q;
  func.params = this;
  gsl_multimin_fminimizer_set( s, &func, minimizer_x, step );

  do {
    iter++;
    status = gsl_multimin_fminimizer_iterate( s );
    
    if ( status ) {
      break;
    }

    size = gsl_multimin_fminimizer_size( s );
    status = gsl_multimin_test_size( size, THRESHOLD_MIN );

    for ( int i = 0 ; i < 2 ; i++ ) {
      Alpha[i] = exp( gsl_vector_get( s->x, i ) );
      Beta[i] = exp( gsl_vector_get( s->x, i+2 ) );
    }

  } while ( ( status == GSL_CONTINUE ) & ( iter < MAX_ITER_MIN ) );

}

void scanModel::estimate_ab_m1 ( ) {

  size_t iter = 0;
  int status;
  double size;

  for ( int i = 0 ; i < 2 ; i++ ) {
    gsl_vector_set( minimizer_x, i, log(Alpha_m1[i]) );
    gsl_vector_set( minimizer_x, (i+2), log(Beta_m1[i]) );
  }

  gsl_vector_set_all( step, 1.0 );
  func.n = 4;
  func.f = model_1_objective;
  func.params = this;
  gsl_multimin_fminimizer_set( s, &func, minimizer_x, step );

  do {
    iter++;
    status = gsl_multimin_fminimizer_iterate( s );
    
    if ( status ) {
      break;
    }

    size = gsl_multimin_fminimizer_size( s );
    status = gsl_multimin_test_size( size, THRESHOLD_MIN );

    //cout << "\t\tEst: ";
    for ( int i = 0 ; i < 2 ; i++ ) {
      Alpha_m1[i] = exp( gsl_vector_get( s->x, i ) );
      Beta_m1[i] = exp( gsl_vector_get( s->x, i+2 ) );
      //cout << Alpha_m1[i] << "\t" << Beta_m1[i] << "\t";
    }
    //cout << "\n";

  } while ( ( status == GSL_CONTINUE ) & ( iter < MAX_ITER_MIN ) );

  if ( iter >= MAX_ITER_MIN ) {
    converge_flag = 0;
  } else {
    converge_flag = 1;
  }
}

void scanModel::EM_algorithm ( ) {

  double start[5];

  double diff = 1;
  double diff2;
  int iter = 0;

  while ( (diff > THRESHOLD_EM) & (iter < MAX_ITER_EM) ) {
    iter++;
    for ( int i = 0 ; i < 2 ; i++ ) {
      start[i] = Alpha[i];
      start[i+2] = Beta[i];
    }
    start[4] = Pi[0];

    expectation( sd->numVarRet( ), sd->numVarRet( )-1 );
    estimate_ab( );
    estimate_pi( sd->numVarRet( ), sd->numVarRet( )-1 );

    diff = 0;
    diff += pow( ( start[0] - Alpha[0] ), 2 ) / pow( ( start[0] ), 2 );
    diff += pow( ( start[1] - Alpha[1] ), 2 ) / pow( ( start[1] ), 2 );
    diff += pow( ( start[2] - Beta[0] ), 2 ) / pow( ( start[2] ), 2 );
    diff += pow( ( start[3] - Beta[1] ), 2 ) / pow( ( start[3] ), 2 );
    diff += pow( ( start[4] - Pi[0] ), 2 ) / pow( ( start[4] ), 2 );
  }
  if ( iter >= MAX_ITER_EM ) {
    converge_flag = 0;
  } else {
    converge_flag = 1;
  }
}

void scanModel::EM_algorithm_pi ( int lower, int upper ) {

  double start[2];

  double diff = 1;
  int iter = 0;

  while ( (diff > THRESHOLD_PI) & (iter < MAX_ITER_PI) ) {
    iter++;
    start[0] = Pi[0];
    start[1] = Pi[1];

    expectation( lower, upper );
    estimate_pi( lower, upper );

    diff = 0;
    diff += pow( ( start[0] - Pi[0] ), 2 );
    diff += pow( ( start[1] - Pi[1] ), 2 );
  }
  if ( iter >= MAX_ITER_PI ) {
    converge_flag = 0;
  } else {
    converge_flag = 1;
  }

}

double scanModel::calculate_LL_model_1_v0 ( int lower, int upper ) {

  // Calculate quantities constant across observations
  ln_beta_func_0_m1 = gsl_sf_lnbeta( Alpha_m1[0], Beta_m1[0] );
  ln_beta_func_1_m1 = gsl_sf_lnbeta( Alpha_m1[1], Beta_m1[1] );

  // Calculate quantities for unique values of observations
  for ( int i = 0 ; i < sd->numUnique( ) ; i++ ) {
    ln_beta_post_0_m1[i] = gsl_sf_lnbeta( Alpha_m1[0] + sd->yu(i), Beta_m1[0] + sd->nu(i) - sd->yu(i) );
    ln_beta_post_1_m1[i] = gsl_sf_lnbeta( Alpha_m1[1] + sd->yu(i), Beta_m1[1] + sd->nu(i) - sd->yu(i) );
  }

  double LL = 0;

  for ( int i = 0 ; i < lower ; i++ ) {
    LL += ln_beta_post_0_m1[sd->map(i)] - ln_beta_func_0_m1;
  }
  for ( int i = lower ; i <= upper ; i++ ) {
    LL += ln_beta_post_1_m1[sd->map(i)] - ln_beta_func_1_m1;
  }
  for ( int i = (upper+1) ; i < sd->numVarRet( ) ; i++ ) {
    LL += ln_beta_post_0_m1[sd->map(i)] - ln_beta_func_0_m1;
  }

  return LL;

}

double scanModel::calculate_LL_model_2 ( int lower, int upper ) {

  double LL = 0;

  for ( int i = 0 ; i < lower ; i++ ) {
    LL += log( Pi[0]*exp(ln_beta_post_0[sd->map(i)] - ln_beta_func_0) + (1-Pi[0])*exp(ln_beta_post_1[sd->map(i)] - ln_beta_func_1) );
  }
  for ( int i = lower ; i <= upper ; i++ ) {
    LL += log( Pi[1]*exp(ln_beta_post_0[sd->map(i)] - ln_beta_func_0) + (1-Pi[1])*exp(ln_beta_post_1[sd->map(i)] - ln_beta_func_1) );
  }
  for ( int i = (upper+1) ; i < sd->numVarRet( ) ; i++ ) {
    LL += log( Pi[0]*exp(ln_beta_post_0[sd->map(i)] - ln_beta_func_0) + (1-Pi[0])*exp(ln_beta_post_1[sd->map(i)] - ln_beta_func_1) );
  }

  return LL;  

}

void scanModel::run_scan ( int permute ) {

  double LL_current_m1_v0;
  double LL_current_m2;

  double Alpha_store[2];
  double Beta_store[2];
  double Pi_store;
  int check_status;
  clock_t cbegin, cend;
  cbegin = clock();

  double LL_temp = -DBL_MAX;
  
  int begin, end;
  if ( permute == 0 ) {
    begin = 1;
    end = 10;
  } else {
    begin = 5;
    end = 6;
  }
  cout << "Begin M2\n";
  for ( int q = 0 ; q < 2 ; q++ ) {
    for ( int i = begin ; i < end ; i+=2 ) {
      if ( q == 0 ) {
	Alpha[0] = 10;
	Alpha[1] = 8;
	Beta[0] = 10;
	Beta[1] = 4;
	Pi[0] = (double)i/10;
      } else {
	Alpha[0] = 10;
	Alpha[1] = .2;
	Beta[0] = 10;
	Beta[1] = .1;
	Pi[0] = (double)i/10;
      }
      EM_algorithm( );   // Initial estimation of parameters
      if ( converge_flag = 1 ) {
	converged[i-1] = 1;
      } else {
	converged[i-1] = 0;
      }
      EM_algorithm_pi( sd->numVarRet(), sd->numVarRet()-1 );
      LL_m1h0 = calculate_LL_model_2( sd->numVarRet(), (sd->numVarRet( )-1) );
      if ( LL_m1h0 > LL_temp ) {
	Alpha_store[0] = Alpha[0];
	Alpha_store[1] = Alpha[1];
	Beta_store[0] = Beta[0];
	Beta_store[1] = Beta[1];
	Pi_store = Pi[0];
	LL_temp = LL_m1h0;
      }
    }
  }
  cout << "End M2\n";
  //win_l = sd->numVarRet( );
  win_l = 0;
  win_u = sd->numVarRet( )-1;

  Alpha_m1[0] = (double)sd->numCases() / 50.0;
  Alpha_m1[1] = 0.8 * (double)sd->numCases() / 50.0;
  Beta_m1[0] = (double)sd->numControls() / 50.0;
  Beta_m1[1] = 0.4 * (double)sd->numControls() / 50.0;
  cout << "Begin M1\n";
  estimate_ab_m1( );
  //cout << Alpha_m1[0] << "\t" << Beta_m1[0] << "\t" << Alpha_m1[1] << "\t" << Beta_m1[1] << "\n";
  if ( converge_flag = 1 ) {
    converged_m1[0] = 1;
  } else {
    converged_m1[0] = 0;
  }
  cout << "End M1\n";
  LL_m1h0_v0 = calculate_LL_model_1_v0( win_l, win_u );
  Pi[0] = Pi_store;
  Alpha[0] = Alpha_store[0];
  Alpha[1] = Alpha_store[1];
  Beta[0] = Beta_store[0];
  Beta[1] = Beta_store[1];
  Alpha_max[0] = Alpha[0];
  Alpha_max[1] = Alpha[1];
  Beta_max[0] = Beta[0];
  Beta_max[1] = Beta[1];

  EM_algorithm_pi( sd->numVarRet(), sd->numVarRet()-1 );
  LL_m1h0 = LL_temp;
  LL_m2h0 = LL_temp;
  LL_max_m1h1_v0[sd->numSizes()] = -DBL_MAX;
  LL_max_m2h1[sd->numSizes()] = -DBL_MAX;
  for ( int i = 0 ; i < sd->numVarRet( ) -1 ; i++ ) {
    posterior[i] = psi[i];
  }
  for ( int j = 0 ; j < sd->numSizes( ) ; j++ ) { // Loop through window sizes
    cout << sd->numWindows(j) << "\n";
  }

  for ( int j = 0 ; j < sd->numSizes( ) ; j++ ) { // Loop through window sizes
    LL_max_m1h1_v0[j] = -DBL_MAX;
    LL_max_m2h1[j] = -DBL_MAX;
    LL_max_ind1_m1_v0[j] = 0;
    LL_max_ind1_m2[j] = 0;
    LL_max_ind2_m1_v0[j] = 0;
    LL_max_ind2_m2[j] = 0;
    cout << j << "\n";
    //Alpha_m1[0] = 10;
    //Alpha_m1[1] = 8;
    //Beta_m1[0] = 10;
    //Beta_m1[1] = 4;
    for ( int i = 0 ; i < sd->numWindows(j) ; i++ ) { // Loop through windows
      Alpha_m1[1] = 0.8 * (double)sd->numCases() / 50.0;
      Beta_m1[1] = 0.4 * (double)sd->numControls() / 50.0;
      win_l = sd->WinStart(j,i);
      win_u = sd->WinEnd(j,i);
      LL_temp = -DBL_MAX;
      for ( int k = 5 ; k < 6 ; k++ ) {
	Pi[1] = (double)k/10;
	Pi[0] = (double)k/10;
	EM_algorithm_pi( win_l, win_u );
	LL_current_m2 = calculate_LL_model_2( win_l, win_u );
	if ( LL_current_m2 > LL_temp ) {
	  Pi_max[0] = Pi[0];
	  Pi_max[1] = Pi[1];
	  Pi_store = Pi[1];
	  LL_temp = LL_current_m2;
	}
      }
      //cend = clock();
      //cout << double(cend-cbegin) << "\n";
      //cout << "\tModel2 End\n";
      //Pi[1] = Pi_store;
      LL_current_m2 = LL_temp;
      //cout << "Start ab_m1 " << i << "\n";
      //cbegin = clock();
      estimate_ab_m1( );
      //cend = clock();
      //cout << double(cend-cbegin) << "\n";
      //cout << "End ab_m1\n\n";
      LL_current_m1_v0 = calculate_LL_model_1_v0( win_l, win_u );
      bayesFactor_m1_v0[j][i] = LL_current_m1_v0 - LL_m1h0_v0;
      bayesFactor_m2[j][i] = LL_current_m2 - LL_m2h0;
      //cout << "\t" << Alpha_m1[0] << "\t" << Beta_m1[0] << "\t" << Alpha_m1[1] << "\t" << Beta_m1[1] << "\t" << bayesFactor_m1_v0[j][i] << "\t" << bayesFactor_m2[j][i]  << "\n";
      // Check status of window
      if ( (Alpha_m1[0]/(Alpha_m1[0]+Beta_m1[0])) < (Alpha_m1[1]/(Alpha_m1[1]+Beta_m1[1])) ) {
	if ( LL_current_m1_v0 > LL_max_m1h1_v0[j] ) {
	  Alpha_m1_max[0] = Alpha_m1[0];
	  Alpha_m1_max[1] = Alpha_m1[1];
	  Beta_m1_max[0] = Beta_m1[0];
	  Beta_m1_max[1] = Beta_m1[1];
	  LL_max_m1h1_v0[j] = LL_current_m1_v0;
	  LL_max_ind1_m1_v0[j] = win_l;
	  LL_max_ind2_m1_v0[j] = win_u;
	}
      } else {
	bayesFactor_m1_v0[j][i] = 0;
      }
      // Check status of window
      if ( (Alpha[0]/(Alpha[0]+Beta[0])) < (Alpha[1]/(Alpha[1]+Beta[1])) ) {
	if ( Pi[0] > Pi[1] ) {
	  check_status = 1;
	} else {
	  check_status = 0;
	}
      } else {
	if ( Pi[0] < Pi[1] ) {
	  check_status = 1;
	} else {
	  check_status = 0;
	}
      }
      if ( (LL_current_m2 > LL_max_m2h1[j]) & (check_status == 1) ) {
	LL_max_m2h1[j] = LL_current_m2;
	LL_max_ind1_m2[j] = win_l;
	LL_max_ind2_m2[j] = win_u;
      } else if (check_status == 0 ) {
	bayesFactor_m2[j][i] = 0;
      }
    }
    if ( LL_max_m1h1_v0[j] > LL_max_m1h1_v0[sd->numSizes()] ) {
      LL_max_m1h1_v0[sd->numSizes()] = LL_max_m1h1_v0[j];
      LL_max_ind1_m1_v0[sd->numSizes()] = LL_max_ind1_m1_v0[j];
      LL_max_ind2_m1_v0[sd->numSizes()] = LL_max_ind2_m1_v0[j];
    }
    if ( LL_max_m2h1[j] > LL_max_m2h1[sd->numSizes()] ) {
      LL_max_m2h1[sd->numSizes()] = LL_max_m2h1[j];
      LL_max_ind1_m2[sd->numSizes()] = LL_max_ind1_m2[j];
      LL_max_ind2_m2[sd->numSizes()] = LL_max_ind2_m2[j];
    }
  }
  cend = clock();
  cout <<  "time = " << double(cend-cbegin) / CLOCKS_PER_SEC << "\n";
}



/************* Accessor methods *****************/

double scanModel::get_alpha( int i ) {
  return Alpha_max[i];
}

double scanModel::get_beta( int i ) {
  return Beta_max[i];
}

double scanModel::get_alpha_m1( int i ) {
  return Alpha_m1_max[i];
}

double scanModel::get_beta_m1( int i ) {
  return Beta_m1_max[i];
}

double scanModel::get_pi( int i ) {
  return Pi_max[i];
}

void scanModel::set( double * a, double * b, double * p ) {

  for ( int i = 0 ; i < 2 ; i++ ) {
    Alpha[i] = a[i];
    Beta[i] = b[i];
  }

  for ( int i = 0 ; i < 2 ; i++ ) {
    Pi[i] = p[i];
  }

}

/************* Constructor *********************/

scanModel::scanModel( scanData * sd1, double * alpha1, double * beta1, double * pi1 ) {

  // Set pointer to scanData object
  sd = sd1;

  // Set parameter vectors
  Alpha = new double[2];
  Beta = new double[2];
  Alpha_m1 = new double[2];
  Beta_m1 = new double[2];
  Pi = new double[2];

  Alpha_max = new double[2];
  Beta_max = new double[2];
  Alpha_m1_max = new double[2];
  Beta_m1_max = new double[2];
  Pi_max = new double[2];

  for ( int i = 0 ; i < 2 ; i++ ) {
    Alpha[i] = alpha1[i];
    Beta[i] = beta1[i];
    Alpha_m1[i] = alpha1[i];
    Beta_m1[i] = beta1[i];
  }

  for ( int i = 0 ; i < 2 ; i++ ) {
    Pi[i] = pi1[i];
  }

  // Functions of parameters
  ln_beta_post_0 = new double[sd->numVariants( )];
  ln_beta_post_1 = new double[sd->numVariants( )]; 
  ln_beta_post_0_m1 = new double[sd->numVariants( )];
  ln_beta_post_1_m1 = new double[sd->numVariants( )];
  ln_pi = new double[2];
  ln_1_minus_pi = new double[2];

  // EM variablts
  psi = new double[sd->numVariants( )];

  // Likelihood values
  LL_max_m1h1_v0 = new double[sd->numSizes() + 1];
  LL_max_m2h1 = new double[sd->numSizes() + 1];
  LL_max_ind1_m1_v0 = new int[sd->numSizes() + 1];
  LL_max_ind1_m2 = new int[sd->numSizes() + 1];
  LL_max_ind2_m1_v0 = new int[sd->numSizes() + 1];
  LL_max_ind2_m2 = new int[sd->numSizes() + 1];
  bayesFactor_m1_v0 = new double*[sd->numSizes()+1];
  bayesFactor_m2 = new double*[sd->numSizes()+1];
  for ( int i = 0 ; i < sd->numSizes() ; i++ ) {
    bayesFactor_m1_v0[i] = new double[2*sd->numVariants()];
    bayesFactor_m2[i] = new double[2*sd->numVariants()];
  }
  posterior = new double[sd->numVariants()];
  converged = new int[10];
  converged_m1 = new int[1];
  converged_perm = new int[10];

  // gsl minimizer variables
  minimizer_x = gsl_vector_alloc( 4 );
  step = gsl_vector_alloc( 4 );
  T = gsl_multimin_fminimizer_nmsimplex2;
  s = gsl_multimin_fminimizer_alloc( T, 4 );

}
